package cz.muni.pv239.wishlist.activities

import android.os.Bundle
import android.util.Log
import android.view.Menu
import android.view.MenuItem
import android.view.View
import com.google.android.material.bottomnavigation.BottomNavigationView
import androidx.appcompat.app.AppCompatActivity
import androidx.core.view.isVisible
import androidx.navigation.NavController
import androidx.navigation.findNavController
import androidx.navigation.ui.AppBarConfiguration
import androidx.navigation.ui.setupActionBarWithNavController
import androidx.navigation.ui.setupWithNavController
import com.google.android.material.snackbar.Snackbar
import cz.muni.pv239.wishlist.R
import cz.muni.pv239.wishlist.database.repository.MyWishlistRepository
import cz.muni.pv239.wishlist.databinding.ActivityMainBinding
import cz.muni.pv239.wishlist.model.mywishlist.MyWishlist
import cz.muni.pv239.wishlist.restclient.MyWishlistService
import cz.muni.pv239.wishlist.restclient.RetrofitUtil
import cz.muni.pv239.wishlist.ui.mywishlist.MyWishlistFragment
import cz.muni.pv239.wishlist.utils.Constants
import cz.muni.pv239.wishlist.utils.NetworkUtils
import cz.muni.pv239.wishlist.utils.SharedPref
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class MainActivity : AppCompatActivity() {

    lateinit var binding: ActivityMainBinding
    private lateinit var navController: NavController
    private lateinit var refreshMyWishlistData: () -> Unit
    private var optionMenu: Menu? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        binding = ActivityMainBinding.inflate(layoutInflater)
        setContentView(binding.root)

        val navView: BottomNavigationView = binding.navView

        navController = findNavController(R.id.nav_host_fragment_activity_main)
        val appBarConfiguration = AppBarConfiguration(
            setOf(
               R.id.navigation_my_wishlist, R.id.navigation_shared_wishlists, R.id.navigation_settings
            )
        )
        setupActionBarWithNavController(navController, appBarConfiguration)
        navView.setupWithNavController(navController)

        navController.addOnDestinationChangedListener { _, destination, _ ->
            var toHideDestination = listOf(
                R.id.navigation_my_wishlist_create_edit,
                R.id.navigation_my_wishlist_share,
                R.id.navigation_shared_wishlists_add,
                R.id.navigation_shared_wishlists_scan,
                R.id.navigation_shared_wishlists_detail)

            binding.navView.isVisible = !toHideDestination.contains(destination.id)

            if (optionMenu != null) {
                optionMenu!!.getItem(0).isVisible = !toHideDestination.contains(destination.id)
            }
        }
    }

    override fun onSupportNavigateUp(): Boolean {
        if (binding.myWishlistLoadingView.visibility == View.VISIBLE) {
            return false
        }
        return navController.navigateUp() || super.onSupportNavigateUp()
    }

    fun changeVisibilityLoadingScreen(isVisible: Boolean) {
        binding.myWishlistLoadingView.visibility = if (isVisible) View.VISIBLE else View.GONE
    }

    fun changeVisibilityOptionMenu(isVisible: Boolean) {
        if (optionMenu != null) {
            optionMenu!!.getItem(0).isVisible = isVisible
        }
    }


    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.main_option_menu, menu)
        if (menu != null) {
            optionMenu = menu
        }
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {

        when (item.itemId) {
            R.id.option_menu_refresh -> {
                when (navController.currentDestination?.id) {
                    R.id.navigation_my_wishlist -> refreshMyWishlistData()
                    R.id.navigation_shared_wishlists -> refreshMyWishlistData()
                    else -> Log.i("IIIIIIIIIIIIII", "onOptionsItemSelected: TODO")
                }
            }
        }

        return super.onOptionsItemSelected(item)
    }

    fun setRefreshMyWishlistData(function: () -> Unit) {
        refreshMyWishlistData = function
    }
}