package cz.muni.pv239.wishlist.database.daos

import androidx.room.Dao
import androidx.room.Delete
import androidx.room.Insert
import androidx.room.OnConflictStrategy
import androidx.room.Query
import androidx.room.Transaction
import cz.muni.pv239.wishlist.database.entities.SharedWishlistEntity
import cz.muni.pv239.wishlist.database.entities.SharedWishlistItemEntity
import cz.muni.pv239.wishlist.database.entities.SharedWishlistWithItems
import okhttp3.internal.cookieToString

@Dao
interface SharedWishlistDao {
    @Insert(onConflict = OnConflictStrategy.REPLACE)
    fun save(entity: SharedWishlistEntity): Long

    @Insert(onConflict = OnConflictStrategy.REPLACE)
    fun save(entity: List<SharedWishlistEntity>)

    @Transaction
    @Delete
    fun delete(entity: SharedWishlistEntity)

    @Transaction
    @Query("DELETE FROM sharedwishlist")
    fun deleteAll()

    @Insert(onConflict = OnConflictStrategy.REPLACE)
    fun save(entity: SharedWishlistItemEntity)

    @Transaction
    fun saveWithItems(entity: SharedWishlistEntity, items: List<SharedWishlistItemEntity>) {
        val idSavedEntity = save(entity)
        for (itemEntity in items) {
            itemEntity.wishlistId = idSavedEntity
            save(itemEntity)
        }
    }

    @Transaction
    @Query("SELECT * FROM sharedwishlist")
    fun getAll(): List<SharedWishlistWithItems>

    @Transaction
    @Query("SELECT * FROM sharedwishlist where idMobile = :id")
    fun getById(id: Long): SharedWishlistEntity

    @Transaction
    @Query("SELECT * FROM sharedwishlist where idMobile = :id")
    fun getByIdWithItems(id: Long): SharedWishlistWithItems
}