package cz.muni.pv239.wishlist.database.repository

import android.content.Context
import cz.muni.pv239.wishlist.database.WishlistDatabase
import cz.muni.pv239.wishlist.database.daos.SharedWishlistDao
import cz.muni.pv239.wishlist.database.entities.SharedWishlistWithItems
import cz.muni.pv239.wishlist.database.repository.mappers.toAppData
import cz.muni.pv239.wishlist.database.repository.mappers.toEntity
import cz.muni.pv239.wishlist.model.sharedwishlist.SharedWishlist

class SharedWishlistRepository(
    context: Context,
    private val dao: SharedWishlistDao =
        WishlistDatabase.create(context).sharedWishlistDao()
) {
    fun saveOrUpdate(sharedWishlist: SharedWishlist) {
        dao.save(sharedWishlist.toEntity())
    }

    fun saveOrUpdate(sharedWishlistList: List<SharedWishlist>) {
        dao.save(sharedWishlistList.map { it.toEntity() })
    }

    fun delete(sharedWishlist: SharedWishlist) {
        dao.delete(sharedWishlist.toEntity())
    }

    fun deleteAll() {
        dao.deleteAll()
    }

    fun saveWithItems(sharedWishlist: SharedWishlist) {
        dao.saveWithItems(sharedWishlist.toEntity(), sharedWishlist.items.map { it.toEntity(0) })
    }

    fun getAll(): List<SharedWishlist> {
        return dao.getAll().map { it.toAppData() }.toMutableList()
    }

    fun getById(id: Long): SharedWishlist {
        return dao.getById(id).toAppData()
    }

    fun getByIdWithItems(id: Long): SharedWishlist {
        return dao.getByIdWithItems(id).toAppData()
    }


}