package cz.muni.pv239.wishlist.restclient

import cz.muni.pv239.wishlist.utils.Constants
import okhttp3.OkHttpClient
import okhttp3.logging.HttpLoggingInterceptor
import retrofit2.Retrofit
import retrofit2.converter.moshi.MoshiConverterFactory
import java.security.SecureRandom
import java.security.cert.X509Certificate
import javax.net.ssl.HostnameVerifier
import javax.net.ssl.SSLContext
import javax.net.ssl.SSLSocketFactory
import javax.net.ssl.TrustManager
import javax.net.ssl.X509TrustManager

object RetrofitUtil {

    private const val BASE_URL = "http://" + Constants.IP_ADDRESS + ":8080/"

    fun createMyWishlistService(): MyWishlistService =
        create(BASE_URL, createOkHttpClient())

    fun createUserWebService(): UserWebService =
        create(BASE_URL, createOkHttpClient())

    fun createSharedItemService(): SharedItemService =
        create(BASE_URL, createOkHttpClient())

    private inline fun <reified T> create(baseUrl: String, okHttpClient: OkHttpClient): T =
        Retrofit.Builder()
            .baseUrl(baseUrl)
            .client(okHttpClient)
            .addConverterFactory(MoshiConverterFactory.create())
            .build()
            .create(T::class.java)

    private fun createOkHttpClient(): OkHttpClient =
        OkHttpClient.Builder().apply {

            var trustAllCerts = arrayOf<TrustManager>(
                object : X509TrustManager {
                    override fun checkClientTrusted(
                        chain: Array<out X509Certificate>?,
                        authType: String?
                    ) {

                    }

                    override fun checkServerTrusted(
                        chain: Array<out X509Certificate>?,
                        authType: String?
                    ) {

                    }

                    override fun getAcceptedIssuers(): Array<X509Certificate> {
                        return arrayOf()
                    }
                }
            )

            val sslContext = SSLContext.getInstance("SSL")
            sslContext.init(null, trustAllCerts, SecureRandom())
            val sslSocketFactory = sslContext.socketFactory

            sslSocketFactory(sslSocketFactory, trustAllCerts[0] as X509TrustManager)

            val logging = HttpLoggingInterceptor().apply {
                level = HttpLoggingInterceptor.Level.BODY
            }

            hostnameVerifier { _, _ -> true }

            addInterceptor(logging)
        }.build()

    fun parseErrorCode(what: String, code: Int): String {
        return when(code) {
            401 -> if (what == "user") "Error: Bad username." else "Error: Permission to $what denied."
            404 -> "Error: $what was not found."
            409 -> "Error: $what already exists."
            422 -> "Error: Synchronization with server. Reload data."
            else -> "Some unknown error occurred"
        }
    }

    fun parseErrorMessage(t: Throwable): String {
        if (t.message?.contains("after 10000") == true) {
            return "Timeout"
        }
        return "Some unknown error occurred"
    }

}