package cz.muni.pv239.wishlist.ui.customviews

import android.content.Context
import android.content.res.TypedArray
import android.text.InputType
import android.util.AttributeSet
import android.view.View
import android.widget.LinearLayout
import androidx.annotation.StyleableRes
import com.google.android.material.textfield.TextInputLayout
import cz.muni.pv239.wishlist.R

class CustomTextInputEditText(context: Context, attrs: AttributeSet?) : LinearLayout(context, attrs) {


    private lateinit var inputEditTextLayout: TextInputLayout

    init {
        initView(context, attrs)
    }

    private fun initView(context: Context, attrs: AttributeSet?) {
        View.inflate(context, R.layout.custom_text_input_edit_text, this)
        val typedArray: TypedArray = context.obtainStyledAttributes(attrs, R.styleable.customTextInputEditText)
        initComponents(typedArray)
        typedArray.recycle()
    }

    private fun initComponents(typedArray: TypedArray) {
        inputEditTextLayout = findViewById(R.id.custom_text_input_layout)
        var index = 0
        while(index < typedArray.indexCount) {
            val attributes = typedArray.getIndex(index)

            if (attributes == R.styleable.customTextInputEditText_hint) {
                if (typedArray.getText(index) == null) {
                    inputEditTextLayout.hint = ""
                } else {
                    inputEditTextLayout.hint = typedArray.getText(index)
                }
            }

            if (attributes == R.styleable.customTextInputEditText_text &&
                typedArray.getText(index) != null) {
                inputEditTextLayout.editText?.setText(typedArray.getText(index))
            }

            if (attributes == R.styleable.customTextInputEditText_isMultiLine) {
                if (typedArray.getBoolean(index, false)) {
                    inputEditTextLayout.editText?.isSingleLine = false
                } else {
                    inputEditTextLayout.editText?.inputType =
                        InputType.TYPE_TEXT_FLAG_NO_SUGGESTIONS
                }
            }

            if (attributes == R.styleable.customTextInputEditText_maxLines) {
                if (typedArray.getInteger(index, -1) != -1) {
                    inputEditTextLayout.editText?.maxLines = typedArray.getInteger(index, -1)
                } else {
                    inputEditTextLayout.editText?.maxLines = 1
                }
            }

            if (attributes == R.styleable.customTextInputEditText_isPassword &&
                typedArray.getBoolean(index, false)) {
                inputEditTextLayout.editText?.inputType = InputType.TYPE_TEXT_VARIATION_PASSWORD
            }
            index++
        }
    }

    fun getInputTextLayout(): TextInputLayout {
        return inputEditTextLayout
    }

}