package cz.muni.pv239.wishlist.ui.login

import android.content.Intent
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.navigation.fragment.findNavController
import com.google.android.material.snackbar.Snackbar
import cz.muni.pv239.wishlist.R
import cz.muni.pv239.wishlist.activities.LogInRegistrationActivity
import cz.muni.pv239.wishlist.activities.MainActivity
import cz.muni.pv239.wishlist.databinding.FragmentLoginBinding
import cz.muni.pv239.wishlist.model.loginregistration.LogIn
import cz.muni.pv239.wishlist.restclient.RetrofitUtil
import cz.muni.pv239.wishlist.restclient.UserWebService
import cz.muni.pv239.wishlist.restclient.response.LogInResponse
import cz.muni.pv239.wishlist.utils.Constants
import cz.muni.pv239.wishlist.utils.SharedPref
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import javax.security.auth.login.LoginException
import kotlin.math.log

class LogInFragment : Fragment() {

    private lateinit var binding: FragmentLoginBinding
    private val prefs: SharedPref by lazy {
        SharedPref(requireContext())
    }
    private val userWebService: UserWebService by lazy {
        RetrofitUtil.createUserWebService()
    }


    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        binding = FragmentLoginBinding.inflate(layoutInflater, container, false)

        initElems()

        return binding.root
    }

    private fun initElems() {

        binding.twRegistration.setOnClickListener {
            findNavController().navigate(R.id.action_navigation_log_in_to_navigation_registration)
        }

        binding.btLogIn.setOnClickListener {
            //TODO validation
            startLogIn(
                binding.tilName.getInputTextLayout().editText?.text.toString(),
                binding.tilPassword.getInputTextLayout().editText?.text.toString()
            )
        }
    }

    private fun startLogIn(
        name: String,
        password: String
    ) {

        if (validate(name, password)) {
            return
        }

        changeVisibilityLoadingScreen(true)

        var logIn = LogIn(
            name,
            password
        )

        userWebService.logIn(logIn).enqueue(
            object : Callback<LogInResponse> {
                override fun onResponse(
                    call: Call<LogInResponse>,
                    response: Response<LogInResponse>
                ) {

                    val loginResponse = response.body()

                    if (response.isSuccessful && loginResponse != null) {


                        Log.i("Aaaaaaa", "onResponse: " + loginResponse)
                        prefs.saveString(Constants.LOG_IN_NAME, name)
                        prefs.saveString(Constants.LOG_IN_PASSWORD, password)
                        prefs.saveString(Constants.USER_TOKEN, loginResponse.token)
                        prefs.saveString(Constants.WISHLIST_TOKEN, loginResponse.tokenWishlist)
                        prefs.saveBool(Constants.ALREADY_LOGGED_IN, true)
                        startMainActivity()

                    } else {
                        val errorMessage = RetrofitUtil.parseErrorCode("user", response.code())
                        Log.e(this::class.simpleName, errorMessage)
                        changeVisibilityLoadingScreen(false)
                        showMessage(errorMessage)
                    }
                }

                override fun onFailure(call: Call<LogInResponse>, t: Throwable) {
                    val errorMessage = RetrofitUtil.parseErrorMessage(t)
                    Log.e(this::class.simpleName, errorMessage, t)
                    changeVisibilityLoadingScreen(false)
                    showMessage(errorMessage)
                }
            }
        )
    }

    private fun validate(name: String, password: String): Boolean {

        if (name.trim() == "") {
            showMessage("Name cannot be empty.")
            return true
        }

        if (name.trim().length > 20) {
            showMessage("Name is too long.")
            return true
        }

        if (password.trim() == "") {
            showMessage("Password cannot be empty.")
            return true
        }

        if (password.trim().length > 30) {
            showMessage("Password is too long.")
            return true
        }
        return false
    }

    override fun onResume() {
        super.onResume()

        if (prefs.getBool(Constants.ALREADY_LOGGED_IN)) {
            startLogIn(
                prefs.getString(Constants.LOG_IN_NAME),
                prefs.getString(Constants.LOG_IN_PASSWORD)
            )
        }
    }

    private fun changeVisibilityLoadingScreen(isVisible: Boolean) {
        (requireActivity() as LogInRegistrationActivity).changeVisibilityLoadingScreen(isVisible)
    }

    private fun showMessage(message: String) {
        Snackbar.make(binding.root, message, Snackbar.LENGTH_SHORT).show()
    }

    private fun startMainActivity() {
        startActivity(Intent(activity, MainActivity::class.java))
        activity?.finish()
    }
}