package cz.muni.pv239.wishlist.ui.mywishlist.adapters

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.Window
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import cz.muni.pv239.wishlist.database.repository.MyWishlistRepository
import cz.muni.pv239.wishlist.databinding.ItemMyWishlistBinding
import cz.muni.pv239.wishlist.model.mywishlist.MyWishlist

class MyWishlistAdapter(
    private val onClickListenerEdit: (MyWishlist) -> Unit
) : ListAdapter<MyWishlist, MyWishlistViewHolder>(MyWishlistDiffUtil()) {

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): MyWishlistViewHolder =
        MyWishlistViewHolder(
            ItemMyWishlistBinding
                .inflate(LayoutInflater.from(parent.context), parent, false),
            onClickListenerEdit
        )

    override fun onBindViewHolder(holder: MyWishlistViewHolder, position: Int) {
        val item = getItem(position)
        holder.bind(item, position)
    }
}

class MyWishlistViewHolder(
    private val binding: ItemMyWishlistBinding,
    private val onClickListenerEdit: (MyWishlist) -> Unit
) : RecyclerView.ViewHolder(binding.root) {

    fun bind(item: MyWishlist, position: Int) {

        binding.myWishlistItemName.text = item.name
        binding.myWishlistItemExpand.setOnClickListener {
            if (binding.myWishlistItemExpand.isSelected) {
                binding.myWishlistItemExpand.isSelected = false
                binding.myWishlistItemExpandableLayout.visibility = View.GONE
            } else {
                binding.myWishlistItemExpand.isSelected = true
                binding.myWishlistItemExpandableLayout.visibility = View.VISIBLE
            }
        }
        binding.myWishlistItemExpandableLayoutIsHidden.isSelected = item._isHidden
        binding.myWishlistItemIsBought.visibility = View.GONE
        binding.myWishlistItemExpandableLayoutDesc.text = item.desc

        binding.myWishlistItemEdit.setOnClickListener {
            onClickListenerEdit(item)
        }
    }
}

class MyWishlistDiffUtil : DiffUtil.ItemCallback<MyWishlist>() {

    override fun areItemsTheSame(oldItem:MyWishlist, newItem: MyWishlist): Boolean =
        oldItem.idMobile == newItem.idMobile

    override fun areContentsTheSame(oldItem: MyWishlist, newItem: MyWishlist): Boolean =
        oldItem == newItem

}