package cz.muni.pv239.wishlist.ui.mywishlist.editcreate

import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.navigation.fragment.findNavController
import androidx.navigation.fragment.navArgs
import com.google.android.material.snackbar.Snackbar
import cz.muni.pv239.wishlist.activities.MainActivity
import cz.muni.pv239.wishlist.database.repository.MyWishlistRepository
import cz.muni.pv239.wishlist.databinding.FragmentCreateEditWishlistBinding
import cz.muni.pv239.wishlist.model.mywishlist.MyWishlist
import cz.muni.pv239.wishlist.restclient.MyWishlistService
import cz.muni.pv239.wishlist.restclient.RetrofitUtil
import cz.muni.pv239.wishlist.utils.Constants
import cz.muni.pv239.wishlist.utils.NetworkUtils
import cz.muni.pv239.wishlist.utils.SharedPref
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class EditCreateFragment : Fragment() {

    private lateinit var binding: FragmentCreateEditWishlistBinding
    private lateinit var pref: SharedPref
    private var wishlist: MyWishlist? = null
    private val args: EditCreateFragmentArgs by navArgs()
    private val myWishlistRepository: MyWishlistRepository by lazy {
        MyWishlistRepository(requireContext())
    }
    private var controlSum: Int = 0
    private var updating: Boolean = false

    private val myWishlistService: MyWishlistService by lazy {
        RetrofitUtil.createMyWishlistService()
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        binding = FragmentCreateEditWishlistBinding.inflate(layoutInflater, container, false)
        pref = SharedPref(requireContext())

        controlSum = args.controlSum

        config()
        initUiElems()

        return binding.root
    }

    private fun initUiElems() {
        wishlist = args.wishlist
        wishlist?.let {
            updating = true
            binding.tilName.getInputTextLayout().editText?.setText(it.name)
            binding.tilDescription.getInputTextLayout().editText?.setText(it.desc)
            binding.btIsHidden.isSelected = it._isHidden
        }

        binding.fbtSave.setOnClickListener {
            saveOrUpdate()
        }

        binding.btIsHidden.setOnClickListener {
            binding.btIsHidden.isSelected = !binding.btIsHidden.isSelected
        }
    }

    private fun saveOrUpdate() {

        if (!canBeDataEdited()) {
            return
        }

        val myWishlistNew = MyWishlist(
            wishlist?.idMobile ?: 0, wishlist?.idServer ?: "",
            binding.tilName.getInputTextLayout().editText?.text.toString(),
            binding.tilDescription.getInputTextLayout().editText?.text.toString(),
            binding.btIsHidden.isSelected,
            wishlist?._isBought ?: false
        )

        changeVisibilityLoadingScreen(true)

        myWishlistService.saveOrUpdateMyWishlist(
            myWishlistNew,
            pref.getInt(Constants.CHECK_SUM_MYWIHSLIST),
            pref.getString(Constants.USER_TOKEN)
        )
            .enqueue(object : Callback<MyWishlist> {
                override fun onResponse(
                    call: Call<MyWishlist>,
                    response: Response<MyWishlist>
                ) {

                    val responseBody = response.body()
                    if (response.isSuccessful && responseBody != null) {

                        response.headers()["Control-Sum"]?.let {
                            pref.saveLong(Constants.CHECK_SUM_MYWIHSLIST, it.toLong())
                        }

                        if (updating) {
                            responseBody.idMobile = myWishlistNew.idMobile
                        }

                        myWishlistRepository.saveOrUpdate(responseBody)
                        findNavController().navigateUp()
                        showMessage("MyWishlist created")
                        changeVisibilityLoadingScreen(false)

                    } else {
                        val errorMessage = RetrofitUtil.parseErrorCode("myWishlist", response.code())
                        Log.e(this::class.simpleName, errorMessage)
                        changeVisibilityLoadingScreen(false)
                        showMessage(errorMessage)
                    }

                }

                override fun onFailure(call: Call<MyWishlist>, t: Throwable) {
                    val errorMessage = RetrofitUtil.parseErrorMessage(t)
                    Log.e(this::class.simpleName, errorMessage, t)
                    changeVisibilityLoadingScreen(false)
                    showMessage(errorMessage)
                }
            })
    }

    private fun config() {
    }

    private fun canBeDataEdited(): Boolean {
        if (!NetworkUtils.isConnected(requireContext())) {
            showMessage("Device is not connected to the internet.")
            return false
        }

        if (validation()) {
            return false
        }

        return true
    }

    private fun validation(): Boolean {
        var name = binding.tilName.getInputTextLayout().editText?.text.toString()
        var desc = binding.tilDescription.getInputTextLayout().editText?.text.toString()

        if (name.trim() == "") {
            showMessage("Name cannot be empty.")
            return true
        }

        if (name.trim().length > 20) {
            showMessage("Name is too long.")
            return true
        }

        if (desc.trim() == "") {
            showMessage("Description cannot be empty.")
            return true
        }

        if (desc.trim().length > 50) {
            showMessage("Description is too long.")
            return true
        }

        return false

    }

    private fun changeVisibilityLoadingScreen(isVisible: Boolean) {
        (requireActivity() as MainActivity).changeVisibilityLoadingScreen(isVisible)
    }

    private fun showMessage(message: String) {
        Snackbar.make(binding.root, message, Snackbar.LENGTH_SHORT).show()
    }


}