package cz.muni.pv239.wishlist.ui.mywishlist.share

import android.content.ClipData
import android.content.ClipboardManager
import android.content.Context
import android.graphics.Bitmap
import android.os.Bundle
import android.text.InputType
import android.text.InputType.TYPE_TEXT_FLAG_CAP_CHARACTERS
import android.util.Base64
import android.util.DisplayMetrics
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.fragment.app.Fragment
import com.google.gson.Gson
import com.google.zxing.BarcodeFormat
import com.journeyapps.barcodescanner.BarcodeEncoder
import cz.muni.pv239.wishlist.databinding.FragmentShareWishlistBinding
import cz.muni.pv239.wishlist.model.mywishlist.SharingWishlistWrapper
import cz.muni.pv239.wishlist.utils.Constants
import cz.muni.pv239.wishlist.utils.SharedPref

class ShareWishlistFragment : Fragment() {

    private lateinit var binding: FragmentShareWishlistBinding
    private lateinit var bitmapHiddenIncluded: Bitmap
    private lateinit var bitmapHiddenExcluded: Bitmap
    private lateinit var encodedStringHiddenExcluded: String
    private lateinit var encodedStringHiddenIncluded: String

    private val pref: SharedPref by lazy {
        SharedPref(requireContext())
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        binding = FragmentShareWishlistBinding.inflate(layoutInflater, container, false)

        generateQrCodes()
        config()
        initUiElems()

        return binding.root
    }

    private fun generateQrCodes() {
        val gson = Gson()
        val displayMetrics = DisplayMetrics()
        activity?.windowManager?.defaultDisplay?.getMetrics(displayMetrics)

        val height = displayMetrics.heightPixels
        val width = displayMetrics.widthPixels

        val encoder = BarcodeEncoder()
        encodedStringHiddenExcluded = Base64.encodeToString(gson.toJson(SharingWishlistWrapper(pref.getString(Constants.WISHLIST_TOKEN), false)).toByteArray(), Base64.DEFAULT)
        encodedStringHiddenIncluded = Base64.encodeToString(gson.toJson(SharingWishlistWrapper(pref.getString(Constants.WISHLIST_TOKEN), true)).toByteArray(), Base64.DEFAULT)
        bitmapHiddenExcluded = encoder.encodeBitmap(encodedStringHiddenExcluded, BarcodeFormat.QR_CODE, width - (width/100*10), height - (height/100*10))
        bitmapHiddenIncluded = encoder.encodeBitmap(encodedStringHiddenIncluded, BarcodeFormat.QR_CODE, width - (width/100*10), height - (height/100*10))
    }

    private fun initUiElems() {
        binding.qrGenerated.setImageBitmap(bitmapHiddenExcluded)
        binding.tilGeneratedCode.getInputTextLayout().editText?.focusable = View.NOT_FOCUSABLE
        setCodeForSharing(encodedStringHiddenExcluded)
        initButtons()
    }

    private fun initButtons() {
        binding.btIncludeHidden.setOnClickListener {

            binding.btIncludeHidden.isSelected = !binding.btIncludeHidden.isSelected
            if (binding.btIncludeHidden.isSelected) {
                binding.qrGenerated.setImageBitmap(bitmapHiddenIncluded)
                setCodeForSharing(encodedStringHiddenIncluded)
            } else {
                binding.qrGenerated.setImageBitmap(bitmapHiddenExcluded)
                setCodeForSharing(encodedStringHiddenExcluded)
            }

        }

        binding.btCopy.setOnClickListener {
            val clipboardManager = activity?.getSystemService(Context.CLIPBOARD_SERVICE) as ClipboardManager
            val clipData = ClipData.newPlainText("text", binding.tilGeneratedCode.getInputTextLayout().editText?.text.toString().trim().replace("\n", ""))
            clipboardManager.setPrimaryClip(clipData)

            Toast.makeText(requireContext(), "Text copied to clipboard", Toast.LENGTH_LONG).show()
        }
    }

    private fun setCodeForSharing(code: String) {
        binding.tilGeneratedCode.getInputTextLayout().editText?.setText(code)
    }

    private fun config() {
    }

}