package cz.muni.pv239.wishlist.ui.registration

import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.fragment.app.setFragmentResult
import androidx.navigation.fragment.findNavController
import com.google.android.material.snackbar.Snackbar
import cz.muni.pv239.wishlist.R
import cz.muni.pv239.wishlist.activities.LogInRegistrationActivity
import cz.muni.pv239.wishlist.activities.MainActivity
import cz.muni.pv239.wishlist.databinding.FragmentRegistrationBinding
import cz.muni.pv239.wishlist.model.loginregistration.Registration
import cz.muni.pv239.wishlist.restclient.MyWishlistService
import cz.muni.pv239.wishlist.restclient.RetrofitUtil
import cz.muni.pv239.wishlist.restclient.UserWebService
import cz.muni.pv239.wishlist.utils.Constants
import cz.muni.pv239.wishlist.utils.SharedPref
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class RegistrationFragment : Fragment() {

    val EMAIL_REGEX = "^[A-Za-z](.*)([@]{1})(.{1,})(\\.)(.{1,})";

    private lateinit var binding: FragmentRegistrationBinding
    private val userWebService: UserWebService by lazy {
        RetrofitUtil.createUserWebService()
    }
    private val prefs: SharedPref by lazy {
        SharedPref(requireContext())
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        binding = FragmentRegistrationBinding.inflate(layoutInflater, container, false)

        initElems()

        return binding.root
    }

    private fun initElems() {

        binding.btRegister.setOnClickListener {

            if (validate()) {
                return@setOnClickListener
            }

            changeVisibilityLoadingScreen(true)

            var registration = Registration(
                binding.tilUsername.getInputTextLayout().editText?.text.toString(),
                binding.tilEmail.getInputTextLayout().editText?.text.toString(),
                binding.tilPassword.getInputTextLayout().editText?.text.toString(),
            )

            userWebService.registration(registration).enqueue(
                object : Callback<Void> {
                    override fun onResponse(
                        call: Call<Void>,
                        response: Response<Void>
                    ) {

                        if (response.isSuccessful) {

                            prefs.saveBool(Constants.ALREADY_LOGGED_IN, false)
                            changeVisibilityLoadingScreen(false)
                            showMessage("registration was successful")
                            findNavController().navigateUp()

                        } else {
                            val errorMessage = RetrofitUtil.parseErrorCode("user", response.code())
                            Log.e(this::class.simpleName, errorMessage)
                            changeVisibilityLoadingScreen(false)
                            showMessage(errorMessage)
                        }
                    }
                    override fun onFailure(call: Call<Void>, t: Throwable) {
                        val errorMessage = RetrofitUtil.parseErrorMessage(t)
                        Log.e(this::class.simpleName, errorMessage, t)
                        changeVisibilityLoadingScreen(false)
                        showMessage(errorMessage)
                    }
                }
            )
        }
    }

    private fun validate(): Boolean {
        var name = binding.tilUsername.getInputTextLayout().editText?.text.toString()
        var email = binding.tilEmail.getInputTextLayout().editText?.text.toString()
        var password = binding.tilPassword.getInputTextLayout().editText?.text.toString()
        var passwordAgain = binding.tilPasswordAgain.getInputTextLayout().editText?.text.toString()

        if (name.trim() == "") {
            showMessage("Name cannot be empty.")
            return true
        }

        if (name.trim().length > 20) {
            showMessage("Name is too long.")
            return true
        }

        if (password.trim() == "") {
            showMessage("Password cannot be empty.")
            return true
        }

        if (password.trim().length > 30) {
            showMessage("Password is too long.")
            return true
        }

        if (password != passwordAgain) {
            showMessage("Passwords are different.")
            return true
        }

        if (!EMAIL_REGEX.toRegex().matches(email)) {
            showMessage("Bad format of email.")
            return true
        }

        return false
    }


    private fun changeVisibilityLoadingScreen(isVisible: Boolean) {
        (requireActivity() as LogInRegistrationActivity).changeVisibilityLoadingScreen(isVisible)
    }

    private fun showMessage(message: String) {
        Snackbar.make(binding.root, message, Snackbar.LENGTH_SHORT).show()
    }
}