package cz.muni.pv239.wishlist.ui.settings

import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import cz.muni.pv239.wishlist.activities.LogInRegistrationActivity
import cz.muni.pv239.wishlist.activities.MainActivity
import cz.muni.pv239.wishlist.database.repository.MyWishlistRepository
import cz.muni.pv239.wishlist.database.repository.SharedWishlistRepository
import cz.muni.pv239.wishlist.databinding.FragmentSettingsBinding
import cz.muni.pv239.wishlist.utils.SharedPref

class SettingsFragment : Fragment() {

    private lateinit var binding: FragmentSettingsBinding
    private val pref: SharedPref by lazy {
        SharedPref(requireContext())
    }
    private val myWishlistRepository: MyWishlistRepository by lazy {
        MyWishlistRepository(requireContext())
    }
    private val sharedWishlistRepository: SharedWishlistRepository by lazy {
        SharedWishlistRepository(requireContext())
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        binding = FragmentSettingsBinding.inflate(layoutInflater, container, false)

        initElems()

        return binding.root
    }

    private fun initElems() {

        binding.btLogOut.setOnClickListener {
            pref.clear()
            myWishlistRepository.deleteAll()
            sharedWishlistRepository.deleteAll()
            startActivity(Intent(activity, LogInRegistrationActivity::class.java))
            requireActivity().finish()
        }

    }

    override fun onResume() {
        super.onResume()
        config()
    }

    private fun config() {
        changeVisibilityOptionMenu(false)
    }

    private fun changeVisibilityOptionMenu(isVisible: Boolean) {
        (requireActivity() as MainActivity).changeVisibilityOptionMenu(isVisible)
    }
}