package cz.muni.pv239.wishlist.ui.sharedwishlists.adapters

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import cz.muni.pv239.wishlist.databinding.ItemMyWishlistBinding
import cz.muni.pv239.wishlist.model.sharedwishlist.SharedItem

class SharedDetailAdapter(
    private val onClickListenerIsClaimed: (SharedItem, Int) -> Unit
) : ListAdapter<SharedItem, SharedItemViewHolder>(SharedItemDiffUtil()) {
    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): SharedItemViewHolder =
        SharedItemViewHolder(
            ItemMyWishlistBinding
                .inflate(LayoutInflater.from(parent.context), parent, false),
            onClickListenerIsClaimed,
        )

    override fun onBindViewHolder(holder: SharedItemViewHolder, position: Int) {
        val item = getItem(position)
        holder.bind(item, position)
    }
}

class SharedItemViewHolder(
    private val binding: ItemMyWishlistBinding,
    private val onClickListenerIsClaimed: (SharedItem, Int) -> Unit
) : RecyclerView.ViewHolder(binding.root) {

    fun bind(item: SharedItem, position: Int) {
        binding.myWishlistItemName.text = item.name
        binding.myWishlistItemExpand.setOnClickListener {
            if (binding.myWishlistItemExpand.isSelected) {
                binding.myWishlistItemExpand.isSelected = false
                binding.myWishlistItemExpandableLayout.visibility = View.GONE
            } else {
                binding.myWishlistItemExpand.isSelected = true
                binding.myWishlistItemExpandableLayout.visibility = View.VISIBLE
            }
        }
        binding.myWishlistItemIsBought.isSelected = item._isBought
        binding.myWishlistItemExpandableLayoutDesc.text = item.desc

        binding.myWishlistItemIsBought.setOnClickListener {
            item._isBought = !item._isBought
            onClickListenerIsClaimed(item, position)
        }

        binding.myWishlistItemEdit.visibility = View.GONE
        binding.myWishlistItemExpandableLayoutIsHidden.visibility = View.GONE
        binding.myWishlistItemExpandableLayoutTextIsHidden.visibility = View.GONE
    }
}

class SharedItemDiffUtil : DiffUtil.ItemCallback<SharedItem>() {
    override fun areItemsTheSame(oldItem: SharedItem, newItem: SharedItem): Boolean =
        oldItem.idMobile == newItem.idMobile

    override fun areContentsTheSame(oldItem: SharedItem, newItem: SharedItem): Boolean =
        oldItem == newItem
}