package cz.muni.pv239.wishlist.ui.sharedwishlists.add

import android.os.Bundle
import java.util.Base64
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.navigation.fragment.findNavController
import com.google.android.material.snackbar.Snackbar
import com.google.gson.Gson
import com.google.gson.JsonSyntaxException

import cz.muni.pv239.wishlist.activities.MainActivity
import cz.muni.pv239.wishlist.databinding.FragmentSharedWishlistsAddBinding
import cz.muni.pv239.wishlist.model.mywishlist.SharingWishlistWrapper
import cz.muni.pv239.wishlist.restclient.RetrofitUtil
import cz.muni.pv239.wishlist.restclient.SharedItemService
import cz.muni.pv239.wishlist.utils.Constants
import cz.muni.pv239.wishlist.utils.SharedPref
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class AddFragment : Fragment() {

    private lateinit var binding: FragmentSharedWishlistsAddBinding
    private val pref: SharedPref by lazy {
        SharedPref(requireContext())
    }
    private val gson: Gson by lazy {
        Gson()
    }
    private val sharedItemService: SharedItemService by lazy {
        RetrofitUtil.createSharedItemService()
    }
    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        binding = FragmentSharedWishlistsAddBinding.inflate(layoutInflater, container, false)

        initUiElems()

        return binding.root
    }

    private fun initUiElems() {

        binding.btSave.setOnClickListener {

            val inputText = binding.tilInsertCode.getInputTextLayout().editText?.text.toString().trim()
            if (inputText == "") {
                showMessage("Inserted code cannot be empty.")
                return@setOnClickListener
            }

            try {

                val decodedString = String(Base64.getDecoder().decode(inputText.replace("\n", "")))
                var sharedWishlistRequest = gson.fromJson(decodedString, SharingWishlistWrapper::class.java)

                addSharedWishlist(sharedWishlistRequest)

            } catch (e: IllegalArgumentException) {
                showMessage("Wrong code inserted.")
                return@setOnClickListener
            } catch (e: JsonSyntaxException) {
                showMessage("Wrong code format.")
                return@setOnClickListener
            }
        }
    }

    private fun addSharedWishlist(sharedWishlistRequest: SharingWishlistWrapper) {
        changeVisibilityLoadingScreen(true)

        sharedItemService.addSharedWishlistToUser(
            sharedWishlistRequest,
            pref.getString(Constants.USER_TOKEN)
        ).enqueue(object : Callback<Void> {
            override fun onResponse(
                call: Call<Void>,
                response: Response<Void>
            ) {

                if (response.isSuccessful) {

                    showMessage("SharedWishlist added")
                    findNavController().navigateUp()
                    changeVisibilityLoadingScreen(false)

                } else {
                    val errorMessage = RetrofitUtil.parseErrorCode("SharedWishlist", response.code())
                    Log.e(this::class.simpleName, errorMessage)
                    changeVisibilityLoadingScreen(false)
                    showMessage(errorMessage)
                }
            }
            override fun onFailure(call: Call<Void>, t: Throwable) {
                val errorMessage = RetrofitUtil.parseErrorMessage(t)
                Log.e(this::class.simpleName, errorMessage, t)
                changeVisibilityLoadingScreen(false)
                showMessage(errorMessage)
            }
        })
    }

    private fun changeVisibilityLoadingScreen(isVisible: Boolean) {
        (requireActivity() as MainActivity).changeVisibilityLoadingScreen(isVisible)
    }

    private fun showMessage(message: String) {
        Snackbar.make(binding.root, message, Snackbar.LENGTH_SHORT).show()
    }


}